function [K1,Rwc1,twc1]=baCam(K,Rwc,twc,U,Xw,method,maxIter)
% optimize camera using fixed U,Xw and initial K, Rwc, twc
opt=[];
switch method
  case 'lm'
    opt=setLM(maxIter);
  case 'gn'
    opt=setGaussNewton(maxIter);
end

if numel(twc)==3
  %function func to optimize
  %x(1:4): k
  %x(5:7): angle-axis representation of rotation
  %x(8:10): translation
  k2K=@(k) ([k(1),0,k(3);0,k(2),k(4);0,0,1]);
  K2k=@(K) ([K(1,1),K(2,2),K(1,3),K(2,3)]/K(3,3));
  func=@(x) pgProjectError(U,k2K(x(1:4)),rot.rodrigues(x(5:7)),x(8:10)',Xw,'v');
  KRt2x=@(K,R,t) [K2k(K)';rot.rodrigues(R);t]';
  
  x0=KRt2x(K,Rwc,twc);
  err_before=norm(func(x0));
  [x,~,~,exitflag]=lsqnonlin(func,x0,[],[],opt);
  err=norm(func(x));
  fprintf('BA: |err|=%f -> %f, exitflag=%d\n',err_before,err,exitflag);
  
  K1=k2K(x(1:4));
  Rwc1=rot.rodrigues(x(5:7));
  twc1=x(8:10)';
elseif numel(twc)==2
  %function func to optimize
  %x(1:2): k,c
  %x(3): rotation angle
  %x(4:5): translation
  k2K=@(k) ([k(1),k(2);0,1]);
  K2k=@(K) ([K(1,1),K(1,2)]/K(2,2));
  func=@(x) pgProjectError(U,k2K(x(1:2)),rot.rodrigues(x(3)),x(4:5)',Xw,'v');
  KRt2x=@(K,R,t) [K2k(K)';rot.rodrigues(R);t]';
  
  x0=KRt2x(K,Rwc,twc);
  err_before=norm(func(x0));
  [x,~,~,exitflag]=lsqnonlin(func,x0,[],[],opt);
  err=norm(func(x));
  fprintf('BA: |err|=%f -> %f, exitflag=%d\n',err_before,err,exitflag);
  
  K1=k2K(x(1:2));
  Rwc1=rot.rodrigues(x(3));
  twc1=x(4:5)';
else
  error('only support 2D or 3D camera!');
end
end
%%%%%%%%%%%%%%%%%%%%%%%%
function opt=setLM(maxIter)
opt=optimset(...
  'Algorithm','levenberg-marquardt'...
  ,'MaxIter',maxIter,'display','off');
end

function opt=setGaussNewton(maxIter)
opt=optimset(...
  'LargeScale','off'...
  ,'LevenbergMarquardt','off'...
  ,'MaxIter',maxIter,'display','off');
end