function [Rwc1,twc1,err]=baCamPose(K,Rwc,twc,U,Xw,method,maxIter)
% optimize 3D/2D camera pose using fixed K,U,Xw and initial Rwc, twc
opt=[];
switch method
  case 'lm'
    opt=setLM(maxIter);
  case 'gn'
    opt=setGaussNewton(maxIter);
end

if numel(twc)==3
  %function func to optimize
  %x(1:3): angle-axis representation of rotation
  %x(4:6): translation
  func=@(x) pgProjectError(U,K,rot.rodrigues(x(1:3)),x(4:6)',Xw,'v');
  Rt2x=@(R,t) [rot.rodrigues(R);t]';
  
  x0=Rt2x(Rwc,twc);
  err_before=norm(func(x0));
  [x,~,~,exitflag]=lsqnonlin(func,x0,[],[],opt);
  err=norm(func(x));
  fprintf('BA: |err|=%f -> %f, exitflag=%d\n',err_before,err,exitflag);
  
  Rwc1=rot.rodrigues(x(1:3));
  twc1=x(4:6)';
elseif numel(twc)==2
  %function func to optimize
  %x(1): rotation angle
  %x(2:3): translation [tx,ty]
  func=@(x) pgProjectError(U,K,rot.rodrigues(x(1)),x(2:3)',Xw,'v');
  Rt2x=@(R,t) [rot.rodrigues(R);t]';
  
  x0=Rt2x(Rwc,twc);
  err_before=norm(func(x0));
  [x,~,~,exitflag]=lsqnonlin(func,x0,[],[],opt);
  err=norm(func(x));
  fprintf('BA: |err|=%f -> %f, exitflag=%d\n',err_before,err,exitflag);
  
  Rwc1=rot.rodrigues(x(1));
  twc1=x(2:3)';
else
  error('only support 2D or 3D camera!');
end
end
%%%%%%%%%%%%%%%%%%%%%%%%
function opt=setLM(maxIter)
opt=optimset(...
  'Algorithm','levenberg-marquardt'...
  ,'MaxIter',maxIter,'display','off');
end

function opt=setGaussNewton(maxIter)
opt=optimset(...
  'LargeScale','off'...
  ,'LevenbergMarquardt','off'...
  ,'MaxIter',maxIter,'display','off');
end