function [Rwc1,twc1]=baCamPoseQ(K,Rwc,twc,U,Xw,method,maxIter)
% optimize camera pose using fixed K,U,Xw and initial Rwc, twc
% Rwc parameterized by quaternion
opt=[];
switch method
  case 'lm'
    opt=setLM(maxIter);
  case 'gn'
    opt=setGaussNewton(maxIter);
end

%function func to optimize
%x(1:4): quaternion
%x(5:7): translation
func=@(x) pgProjectError(U,K,rot.q2R(x(1:4)),x(5:7)',Xw,'v');
Rt2x=@(R,t) [rot.R2q(R);t]';

x0=Rt2x(Rwc,twc);
fprintf('Before BA: |err|=%f\n',norm(func(x0)));
[x,~,~,exitflag,~,~,J]=lsqnonlin(func,x0,[],[],opt);
fprintf('After  BA: |err|=%f, exitflag=%d\n',norm(func(x)),exitflag);

Rwc1=rot.q2R(x(1:4));
twc1=x(5:7)';
end
%%%%%%%%%%%%%%%%%%%%%%%%
function opt=setLM(maxIter)
opt=optimset(...
  'Algorithm','levenberg-marquardt'...
  ,'MaxIter',maxIter);
end

function opt=setGaussNewton(maxIter)
opt=optimset(...
  'LargeScale','off'...
  ,'LevenbergMarquardt','off'...
  ,'MaxIter',maxIter);
end