function [K1,Rwc1,twc1]=baCamQ(K,Rwc,twc,U,Xw,method,maxIter)
% optimize camera using fixed U,Xw and initial K, Rwc, twc
% using quaternion for rotation
opt=[];
switch method
  case 'lm'
    opt=setLM(maxIter);
  case 'gn'
    opt=setGaussNewton(maxIter);
end

%function func to optimize
%x(1:4): k
%x(5:8): quaternion
%x(9:11): translation
k2K=@(k) ([k(1),0,k(3);0,k(2),k(4);0,0,1]);
K2k=@(K) ([K(1,1),K(2,2),K(1,3),K(2,3)]/K(3,3));
func=@(x) pgProjectError(U,k2K(x(1:4)),rot.q2R(x(5:8)),x(9:11)',Xw,'v');
KRt2x=@(K,R,t) [K2k(K)';rot.R2q(R);t]';

x0=KRt2x(K,Rwc,twc);
fprintf('Before BA: |err|=%f\n',norm(func(x0)));
[x,~,~,exitflag,~,~,J]=lsqnonlin(func,x0,[],[],opt);
fprintf('After  BA: |err|=%f, exitflag=%d\n',norm(func(x)),exitflag);

K1=k2K(x(1:4));
Rwc1=rot.q2R(x(5:8));
twc1=x(9:11)';
end
%%%%%%%%%%%%%%%%%%%%%%%%
function opt=setLM(maxIter)
opt=optimset(...
  'Algorithm','levenberg-marquardt'...
  ,'MaxIter',maxIter);
end

function opt=setGaussNewton(maxIter)
opt=optimset(...
  'LargeScale','off'...
  ,'LevenbergMarquardt','off'...
  ,'MaxIter',maxIter);
end