classdef cMultiGaussian < handle
  % multi-gaussian distribution object
  % u<dimx1>: mean
  % P<dimxdim>: covariance matrix
  % dim<1x1>: dimention
  % A<dimxdim>: A*A'=P, for sampling
  % I<dimxdim>: I=inv(P), for plotting
  properties
    u=[];
    P=[];
    dim=0;
    
    A=[]; %P=A*A'
    I=[]; %I=inv(P), information matrix
  end
  methods
    function obj=cMultiGaussian(u,P)
      %construction multi-gaussian object with mean u and covariance P
      d = size(u,1);
      assert(d==size(P,1) && d==size(P,2));
      obj.u = u;
      obj.P = P;
      obj.dim = d;
      
      obj.A=chol(P,'lower');
      obj.I=inv(P);
    end
    
    function X=sample(obj,n)
      % sample from the multi-gaussian distribution
      X=randn(obj.dim,n);
      X=obj.A*X+repmat(obj.u,[1,n]);
    end
    
    %     function pr=probability(chi2)
    %       %return the probability of sample within chi2 error
    %       pr=1-exp(-0.5*chi2);
    %     end
    
    function c2=chi2error(obj,x)
      %chi2 error from x to u, i.e. square of mahalonobis distance
      assert(size(x,1)==obj.dim && size(x,2)==1);
      c2=(x-obj.u)'*obj.I*(x-obj.u);
    end
    
    function plot(obj,r,varargin)
      if obj.dim==2 || obj.dim==3
        vz.ellipsoid(obj.u,obj.I,r,varargin{:});
      elseif obj.dim==1
        sig=sqrt(obj.P);
        x=linspace(-5*sig,5*sig,1000);
        y=exp(-(x-obj.u).^2./(2*obj.P))/(sig*sqrt(2*pi));
        if exist('r','var')~=1
          r='b';
        end
        plot(x,y,r,varargin{:});
      elseif obj.dim>3
        warning('matlib:warn','can not plot dimention larger than 3');
      end
    end
  end
  methods(Static)
    function obj=estimate(X)
      % estimate multi-gaussian distribution from sample set X
      obj=cMultiGaussian(mean(X,2),cov(X'));
    end
  end
end