function [Rwc,twc,U]=simRandomCamera(K,Xw,w,h)
% simulate a random camera K*[Rwc,twc] such that Xw are all inside view
% defined by (0...w]x(0...h]
% K<3x3>: calibration matrix, unit is pixel
% Xw<3xn>: world 3d points, unit is interpreted by user
% w<1x1>,h<1x1>: image size, unit is pixel
% Rwc<3x3>,twc<3x1>: output camera pose, twc's unit is the same as Xw
% U<2xn>: output projected 2d image points, unis is pixel
assert(size(Xw,1)==3);
[L,C]=boundingSphere(Xw);
D=max(K(1,1)*L/w,K(2,2)*L/h);
for cnt=1:50000
  Rwc=rot.v2R([0;0;1],randUnitSpherePoint(1,[-0.5,-1]))*rot.e2R(randin(0,2*pi),'z');
  twc=[randin(-L/2,L/2,[2,1]);randin(0.8,2)*D]-Rwc*C;
  P=K*[Rwc,twc];
  U=pgProject(P,Xw);
  Ux=U(1,:);
  Uy=U(2,:);
  if ~any(Ux<=0 | Ux>w) && ~any(Uy<=0 | Uy>h)
    break;
  else
    Rwc=[];
    twc=[];
    U=[];
  end
end
end

function [L,C]=boundingSphere(Xw)
maxXw=max(Xw,[],2);
minXw=min(Xw,[],2);
C=(maxXw+minXw)/2;%center
L=norm(maxXw-minXw);%diameter
end